function [W,W_iter,iter, J_ICA_iter]=MLICA_scaledNG(W_init, X, c_v, gam_v, maxIterICA)
%perform noncircular complex ICA based on a noncircular complex GGD model
%W_init: MxM matrix used as initialization for the demixing matrix W
%X: MxL matrix of mixture signals
%c_v: Mx1 vector of shape parameters for GGD
%gam_v: Mx1 vector of noncircularity indices for GGD
%maxIterICA: maximum number of iterations

M=size(X,1);
N=M;
L=size(X,2);

%if change in W is below N*tol iteration is stopped
tol=1e-8;

%inital step size
u0=1e-2;
u=u0;


%expand vector of shape parameters and vector of noncircularity indices
%such that we can us MATLABs elementwise multiplication later on
c=c_v(ones(L,1),:).';
gam=gam_v(ones(L,1),:).';

%precalculate some values to save computations
gamm_=1/2*gamma(2./c)./gamma(1./c);
const=2*c.*gamm_.^c./((gam.^2-1).^c);

%preallocate arrays
J_ICA_iter=zeros(maxIterICA,1);
W_iter=zeros(M,M,maxIterICA);

iter=1;
%inital separation with W=W_init and cost function calculation
W=W_init;
y=W*X;
J_ICA_iter(1)=real((-log(abs(det(W*W')))+sum(mean((gamm_./(gam.^2-1).*(-gam.*(y.^2+conj(y).^2)+2.*y.*conj(y))).^(c),2))));
while (iter<maxIterICA)
    
    %SEPARATED COMPONENTS ESTIMATION
    y=W*X;
    
    %noncircular GGD score function
    %expNonLinear=const.*(2.*gam.*(yr.^2-yi.^2)-2.*(yr.^2+yi.^2)).^(c-1).*(gam.*conj(y)-y);
    expNonLinear=const.*(gam.*(y.^2+conj(y).^2)-2.*y.*conj(y)).^(c-1).*(gam.*conj(y)-y);
    
    gradAverage=(expNonLinear*y')/L;
    
    %NEW ESTIMATED MATRIX
    tmpMat=eye(M)-gradAverage;
    grad=tmpMat*W;
    
    %perform row normalization (L2-norm) of gradient matrix
    %Strictly speaking a matrix decoupling method such as
    %M. Anderson, X.-L. Li, P. A. Rodriquez, and T. Adali. "An Effective Decoupling Method for Matrix Optimization and its Application to the ICA Problem", Proc. IEEE ICASSP 2012
    %would need to be applied prior to the row normalization.
    %However, in our experiemtns the direct row normalization of the gradient matrix works well and greatly improves performance especially for sources with different noncircularity and/or shape parameters
    grad=diag(1./sqrt(sum(abs(grad).^2,2)))*grad;
    newW=(W+u.*grad);
    
    %stop iteration if no significant change
    if (norm(newW/W-eye(N), 'fro')<N*tol)
        break;
    end
    
    %calculate new cost function value
    newcost=real((-log(abs(det(newW*newW')))+sum(mean((gamm_./(gam.^2-1).*(-gam.*(y.^2+conj(y).^2)+2.*y.*conj(y))).^(c),2))));
    
    %decrease step size if cost function improves by less than 1e-5 --> we are near a (local) optimum
    if iter>1
        if abs(newcost-J_ICA_iter(iter-1))<1e-5
            u=0.9*u;
        end
    end
    J_ICA_iter(iter+1)=newcost;
    W=newW;
    W_iter(:,:,iter)=W;
    
    iter=iter+1;
    
end
